searchData={"content_type":"text/markdown","items":[{"doc":"Provides a set of helpers to work with Changesets.","ref":"ChangesetHelpers.html","title":"ChangesetHelpers","type":"module"},{"doc":"Adds an error to the nested changeset.\n\n```elixir\naccount_changeset =\n  ChangesetHelpers.add_error(account_changeset, [:user, :articles, :error_key], \"Some error\")\n```","ref":"ChangesetHelpers.html#add_error/4","title":"ChangesetHelpers.add_error/4","type":"function"},{"doc":"","ref":"ChangesetHelpers.html#change_assoc/2","title":"ChangesetHelpers.change_assoc/2","type":"function"},{"doc":"Returns the nested association in a changeset. This function will first look into the changes and then fails back on\ndata wrapped in a changeset.\n\nChanges may be added to the given changeset through the third argument.\n\nA tuple is returned containing the root changeset, and the changeset of the association.\n\n```elixir\n{account_changeset, address_changeset} =\n  change_assoc(account_changeset, [:user, :config, :address], %{street: \"Foo street\"})\n```","ref":"ChangesetHelpers.html#change_assoc/3","title":"ChangesetHelpers.change_assoc/3","type":"function"},{"doc":"Returns the nested association in a changeset at the given index.\n\nA tuple is returned containing the root changeset, the changesets of the association and the changeset at the\nspecified index.\n\nSee `change_assoc(struct_or_changeset, keys, changes)`.\n```","ref":"ChangesetHelpers.html#change_assoc/4","title":"ChangesetHelpers.change_assoc/4","type":"function"},{"doc":"This function allows checking if a given field is different between two changesets.\n\n```elixir\n{street_changed, street1, street2} =\n  diff_field(account_changeset, new_account_changeset, [:user, :config, :address, :street])\n```","ref":"ChangesetHelpers.html#diff_field/3","title":"ChangesetHelpers.diff_field/3","type":"function"},{"doc":"","ref":"ChangesetHelpers.html#do_has_change?/2","title":"ChangesetHelpers.do_has_change?/2","type":"function"},{"doc":"Fetches a nested change from the given changeset.\n\nThis function only looks at the `:changes` field of the given `changeset` and returns `{:ok, value}` if the change is\npresent or `:error` if it's not.\n\n```elixir\n{:ok, street} =\n  ChangesetHelpers.fetch_change(account_changeset, [:user, :config, :address, :street])\n```","ref":"ChangesetHelpers.html#fetch_change/2","title":"ChangesetHelpers.fetch_change/2","type":"function"},{"doc":"Same as `fetch_change/2` but returns the value or raises if the given nested key was not found.\n\n```elixir\nstreet = ChangesetHelpers.fetch_change!(account_changeset, [:user, :config, :address, :street])\n```","ref":"ChangesetHelpers.html#fetch_change!/2","title":"ChangesetHelpers.fetch_change!/2","type":"function"},{"doc":"Fetches the given nested field from changes or from the data.\n\nWhile `fetch_change/2` only looks at the current changes to retrieve a value, this function looks at the changes and\nthen falls back on the data, finally returning `:error` if no value is available.\n\nFor relations, these functions will return the changeset original data with changes applied. To retrieve raw\nchangesets, please use `fetch_change/2`.\n\n```elixir\n{:changes, street} =\n  ChangesetHelpers.fetch_field(account_changeset, [:user, :config, :address, :street])\n```","ref":"ChangesetHelpers.html#fetch_field/2","title":"ChangesetHelpers.fetch_field/2","type":"function"},{"doc":"Same as `fetch_field/2` but returns the value or raises if the given nested key was not found.\n\n```elixir\nstreet = ChangesetHelpers.fetch_field!(account_changeset, [:user, :config, :address, :street])\n```","ref":"ChangesetHelpers.html#fetch_field!/2","title":"ChangesetHelpers.fetch_field!/2","type":"function"},{"doc":"","ref":"ChangesetHelpers.html#field_fails_validation?/3","title":"ChangesetHelpers.field_fails_validation?/3","type":"function"},{"doc":"","ref":"ChangesetHelpers.html#field_violates_constraint?/3","title":"ChangesetHelpers.field_violates_constraint?/3","type":"function"},{"doc":"","ref":"ChangesetHelpers.html#has_change?/2","title":"ChangesetHelpers.has_change?/2","type":"function"},{"doc":"Puts the given nested association in the changeset through a given list of field names.\n\n```elixir\nChangesetHelpers.put_assoc(account_changeset, [:user, :config, :address], address_changeset)\n```\n\nInstead of giving a Changeset or a schema as the third argument, a function may also be given receiving the nested\nChangeset(s) to be updated as argument.\n\n```elixir\nChangesetHelpers.put_assoc(account_changeset, [:user, :articles],\n  &(Enum.concat(&1, [%Article{} |> Ecto.Changeset.change()])))\n```\n\nIn the code above, we add a new empty Article to the articles association (typically done when we want to add a new\narticle to a form).","ref":"ChangesetHelpers.html#put_assoc/3","title":"ChangesetHelpers.put_assoc/3","type":"function"},{"doc":"Puts the given nested association in the changeset at the given index.\n\nSee `put_assoc(changeset, keys, value)`.","ref":"ChangesetHelpers.html#put_assoc/4","title":"ChangesetHelpers.put_assoc/4","type":"function"},{"doc":"","ref":"ChangesetHelpers.html#update_assoc_changes/3","title":"ChangesetHelpers.update_assoc_changes/3","type":"function"},{"doc":"Works like `Ecto.Changeset.validate_change/3` but may receive multiple fields.\n\nThe `validator` function receives as argument a keyword list, where the keys are the field\nnames and the values are the change for this field, or the data.any()\n\nIf one of the fields is `nil`, the `validator` function is not invoked.","ref":"ChangesetHelpers.html#validate_changes/4","title":"ChangesetHelpers.validate_changes/4","type":"function"},{"doc":"Validates the result of the comparison of\n  * two fields (where at least one is a change) or\n  * a change and a value, where the value is an integer, a `Date`, a `Time`,\n    a `DateTime` or a `NaiveDateTime`.","ref":"ChangesetHelpers.html#validate_comparison/5","title":"ChangesetHelpers.validate_comparison/5","type":"function"},{"doc":"* `:error_on_field` - specifies on which field to add the error, defaults to the first field\n  * `:message` - a customized message on failure","ref":"ChangesetHelpers.html#validate_comparison/5-options","title":"Options - ChangesetHelpers.validate_comparison/5","type":"function"},{"doc":"Validates a list of values using the given validator.\n\n```elixir\nchangeset =\n  %Appointment{}\n  |> Appointment.changeset(%{days_of_week: [1, 3, 8]})\n  |> validate_list(:days_of_week, &Ecto.Changeset.validate_inclusion/3, [1..7])\n\nassert [days_of_week: {\"is invalid\", [validation: :list, index: 2, validator: :validate_inclusion]}] = changeset.errors\nassert [days_of_week: {:list, [validator: :validate_inclusion]}] = changeset.validations\n```","ref":"ChangesetHelpers.html#validate_list/4","title":"ChangesetHelpers.validate_list/4","type":"function"},{"doc":"`validate_not_present/3` checks if the specified attribute is present in the changeset's parameters.\nIf the attribute is present, an error is added to the changeset's `:errors` key.","ref":"ChangesetHelpers.html#validate_not_present/2","title":"ChangesetHelpers.validate_not_present/2","type":"function"},{"doc":"# ChangesetHelpers\r\n\r\nThis library provides a set of helper functions to work with Ecto Changesets.\r\n\r\n### `validate_comparison(changeset, field1, operator, field2_or_value, opts)`\r\n\r\nValidates the result of the comparison of\r\n  * two fields (where at least one is a change) or\r\n  * a change and a value, where the value is an integer, a `Date`, a `Time`,\r\n    a `DateTime` or a `NaiveDateTime`.\r\n\r\n```elixir\r\nvalidate_comparison(changeset, :start_time, :lt, :end_time)\r\n\r\nassert [start_time: {\"must be less than 10:00:00\", [validation: :comparison]}] = changeset.errors\r\nassert [start_time: :comparison, end_time: :comparison] = changeset.validations\r\n```\r\n\r\n```elixir\r\nvalidate_comparison(appointment_changeset, :end_time, :lt, ~T[21:00:00])\r\n\r\nassert [end_time: {\"must be less than 21:00:00\", [validation: :comparison]}] = changeset.errors\r\nassert [end_time: :comparison] = changeset.validations\r\n```\r\n\r\n### `validate_not_present(changeset, field)`\r\n\r\nChecks if the specified attribute is present in the changeset's parameters. If the attribute is present, an error is added to the changeset's `:errors` key.\r\n\r\n```elixir\r\nchanges = %{foo: 1, bar: 2}\r\n\r\nchangeset =\r\n  %Appointment{}\r\n  |> cast(changes, [:foo])\r\n  |> validate_not_present([:bar])\r\n  |> validate_not_present([:baz, :qux])\r\n\r\nrefute changeset.valid?\r\nassert [bar: {\"cannot be changed\", [validation: :not_present]}] = changeset.errors\r\n```\r\n\r\n### `validate_list(changeset, field, validation_fun, validation_fun_args)`\r\n\r\nValidates a list of values using the given validator.\r\n\r\n```elixir\r\nchangeset =\r\n  %Appointment{}\r\n  |> Appointment.changeset(%{days_of_week: [1, 3, 8]})\r\n  |> validate_list(:days_of_week, &Ecto.Changeset.validate_inclusion/3, [1..7])\r\n\r\nassert [days_of_week: {\"is invalid\", [validation: :list, index: 2, validator: :validate_inclusion]}] = changeset.errors\r\nassert [days_of_week: {:list, [validator: :validate_inclusion]}] = changeset.validations\r\n```\r\n\r\nAs the validator function is from the `Ecto.Changeset` module, you may just write `:validate_inclusion`.\r\n\r\n### `put_assoc(changeset, keys, value)`\r\n\r\nPuts the given nested association in the changeset through a given list of field names.\r\n\r\n```elixir\r\nChangesetHelpers.put_assoc(account_changeset, [:user, :config, :address], address_changeset)\r\n```\r\n\r\nInstead of giving a Changeset or a schema as the third argument, a function may also be given in order to modify the\r\nnested changeset in one go.\r\n\r\n```elixir\r\nChangesetHelpers.put_assoc(account_changeset, [:user, :articles],\r\n  &(Enum.concat(&1, [%Article{} |> Ecto.Changeset.change()])))\r\n```\r\n\r\nIn the code above, we change a new empty Article, and add the changeset into the articles association (typically done when we want to add a new\r\nrow of form inputs to add an entity into a form handling a nested collection of entities).\r\n\r\n### `put_assoc(changeset, keys, index, value)`\r\n\r\nPuts the given nested association in the changeset through a given list of field names, at the given index.\r\n\r\n### `change_assoc(struct_or_changeset, keys, changes \\\\ %{})`\r\n\r\nReturns the nested association in a changeset. This function will first look into the changes and then fails back on\r\ndata wrapped in a changeset.\r\n\r\nChanges may be added to the given changeset through the third argument.\r\n\r\nA tuple is returned containing the modified root changeset and the changeset of the association.\r\n\r\n```elixir\r\n{account_changeset, address_changeset} =\r\n  change_assoc(account_changeset, [:user, :user_config, :address], %{street: \"Foo street\"})\r\n```\r\n\r\n### `change_assoc(struct_or_changeset, keys, index, changes \\\\ %{})`\r\n\r\nReturns the nested association in a changeset at the given index.\r\n\r\n### `update_assoc_changes(changeset, keys, changes)`\r\n\r\nAdds changes to the changed association changesets.\r\n\r\n### `fetch_field(changeset, keys)`\r\n\r\nFetches the given nested field from changes or from the data.\r\n\r\n```elixir\r\n{:changes, street} =\r\n  ChangesetHelpers.fetch_field(account_changeset, [:user, :config, :address, :street])\r\n```\r\n\r\n### `fetch_field!(changeset, keys)`\r\n\r\nSame as `fetch_field/2` but returns the value or raises if the given nested key was not found.\r\n\r\n```elixir\r\nstreet = ChangesetHelpers.fetch_field!(account_changeset, [:user, :config, :address, :street])\r\n```\r\n\r\n### `fetch_change(changeset, keys)`\r\n\r\nFetches the given nested field from changes or from the data.\r\n\r\n```elixir\r\n{:ok, street} =\r\n  ChangesetHelpers.fetch_change(account_changeset, [:user, :config, :address, :street])\r\n```\r\n\r\n### `fetch_change!(changeset, keys)`\r\n\r\nSame as `fetch_change/2` but returns the value or raises if the given nested key was not found.\r\n\r\n```elixir\r\nstreet = ChangesetHelpers.fetch_change!(account_changeset, [:user, :config, :address, :street])\r\n```\r\n\r\n### `has_change?(changeset, keys)`\r\n\r\n```elixir\r\nChangesetHelpers.has_change?(account_changeset, [:user, :config, :address, :street])\r\n```\r\n\r\n### `diff_field(changeset1, changeset2, keys)`\r\n\r\nThis function allows checking if a given field is different in two changesets.\r\n\r\n```elixir\r\n{street_changed, street1, street2} =\r\n  diff_field(account_changeset, new_account_changeset, [:user, :user_config, :address, :street])\r\n```\r\n\r\n### `add_error(changeset, keys, message, extra \\\\ [])`\r\n\r\nAdds an error to the nested changeset.\r\n\r\n```elixir\r\nChangesetHelpers.add_error(account_changeset, [:user, :articles, :error_key], \"Some error\")\r\n```\r\n\r\n### `field_fails_validation?(changeset, field, validations)`\r\n\r\nChecks whether a field as the given validation error key.\r\n\r\n```elixir\r\nChangesetHelpers.field_fails_validation?(changeset, :email, :unsafe_unique)\r\n```\r\n\r\n### `field_violates_constraint?(changeset, field, constraints)`\r\n\r\nChecks whether a field as the given constraint error key.\r\n\r\n```elixir\r\nChangesetHelpers.field_violates_constraint?(changeset, :email, :unique)\r\n```\r\n\r\n### `validate_changes(changeset, fields, meta, validator)`\r\n\r\nWorks like `Ecto.Changeset.validate_change/3` but may receive multiple fields.","ref":"readme.html","title":"ChangesetHelpers","type":"extras"},{"doc":"Add `changeset_helpers` for Elixir as a dependency in your `mix.exs` file:\r\n\r\n```elixir\r\ndef deps do\r\n  [\r\n    {:changeset_helpers, \"~> 0.21.0\"}\r\n  ]\r\nend\r\n```","ref":"readme.html#installation","title":"Installation - ChangesetHelpers","type":"extras"},{"doc":"HexDocs documentation can be found at [https://hexdocs.pm/changeset_helpers](https://hexdocs.pm/changeset_helpers).","ref":"readme.html#hexdocs","title":"HexDocs - ChangesetHelpers","type":"extras"}]}